==> portrac-0.1b.tar.bz2: portrac-0.1b/PortChecker.cc <==
/* Copyright 2009 e-soul.org. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *      the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *      and the following disclaimer in the documentation and/or other materials provided with the 
 *      distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY E-SOUL.ORG ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL E-SOUL.ORG 
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those of the authors and 
 * should not be interpreted as representing official policies, either expressed or implied, of e-soul.org.
 */

#include <unistd.h>
#include <stdlib.h>
#include <sstream>
#include <string>
#include <algorithm>

#include <QStringList>

#include "PortChecker.hh"
#include "Preferences.hh"

using namespace std;

PortChecker::PortChecker(Preferences* preferences)
{
	this->preferences = preferences;
	pipeFileDescr[0] = -1;
	pipeFileDescr[1] = -1;
}

PortChecker::~PortChecker()
{
}

QStringList* PortChecker::check()
{
	int status = pipe(pipeFileDescr);
	if(status != 0)
	{
		perror("pipe");
		return NULL;
	}
	QStringList* list = new QStringList;
	switch(fork())
	{
		case -1: // fork failed
			return list;
		case 0: // child
			close(pipeFileDescr[0]);
			dup2(pipeFileDescr[1], 1);
			executePkgVersion();
		default: // parent
			close(pipeFileDescr[1]);
			catchPkgVersionOutput(list);
	}
	close(pipeFileDescr[0]);
	close(pipeFileDescr[1]);
	return list;
}

void PortChecker::executePkgVersion()
{
	const char* index;
	if(isCompressedIndex())
	{
		index = handleCompressedIndex();
	}
	else
	{
		index = (preferences->getPreference(INDEX_PREF).c_str());
	}
	execl("/usr/sbin/pkg_version", "pkg_version", "-I", index, (char *) 0);
	perror("execl");
}

void PortChecker::catchPkgVersionOutput(QStringList* list)
{
	int n, i;
	char buff[BUFF_LEN] = {'\0'};
	string* s = new string;

	while((n = read(pipeFileDescr[0], buff, BUFF_LEN)) > 0)
	{
		for(i = 0; i < n; i++)
		{
			if(buff[i] == '\n')
			{
				if(s->find("<") != string::npos)
				{
					size_t endpos = s->find_last_not_of(" \t");
					s->erase(endpos);
					list->append(s->c_str());
				}
				s->clear();
			}
			else
			{
				s->append(1, buff[i]);
			}
		}
	}

	delete s;
}

bool PortChecker::isCompressedIndex()
{
	string index = preferences->getPreference(INDEX_PREF);
	if(index.find(".bz2", index.find_last_of("/")) != string::npos)
	{
		return TRUE;
	}
	return FALSE;
}

const char* PortChecker::handleCompressedIndex()
{
	char* home = getenv("HOME");
	string* index = new string(home);
	*index += "/.portracData";
	ostringstream oss("/bin/mkdir -p ", ios_base::app);
	oss << index->c_str();
	oss << " && /usr/bin/cd " << index->c_str();
	oss << " && /usr/bin/fetch -o "<< index->c_str() << "/INDEX.bz2 " << preferences->getPreference(INDEX_PREF).c_str();
	oss << " && /usr/bin/bunzip2 "<< index->c_str() << "/INDEX.bz2";
	system(oss.str().c_str());
	*index += "/INDEX";
	const char* cindex = index->c_str();
	delete index;
	return cindex;
}

void PortChecker::run()
{
	list = check();
}

QStringList* PortChecker::getList()
{
	return list;
}

==> portrac-0.2.tar.bz2: portrac-0.2/PortChecker.cc <==
/* Copyright 2009 e-soul.org. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 * that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *      the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *      and the following disclaimer in the documentation and/or other materials provided with the 
 *      distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY E-SOUL.ORG ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL E-SOUL.ORG 
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those of the authors and 
 * should not be interpreted as representing official policies, either expressed or implied, of e-soul.org.
 */

#include <unistd.h>
#include <stdlib.h>
#include <sstream>
#include <string>
#include <algorithm>

#include <QStringList>

#include "PortChecker.hh"
#include "Preferences.hh"

using namespace std;

PortChecker::PortChecker(Preferences* preferences)
{
	this->preferences = preferences;
	pipeFileDescr[0] = -1;
	pipeFileDescr[1] = -1;
}

PortChecker::~PortChecker()
{
}

QStringList* PortChecker::check()
{
	int status = pipe(pipeFileDescr);
	if(status != 0)
	{
		perror("pipe");
		return NULL;
	}
	QStringList* list = new QStringList;
	switch(fork())
	{
		case -1: // fork failed
			return list;
		case 0: // child
			close(pipeFileDescr[0]);
			dup2(pipeFileDescr[1], 1);
			executePkgVersion();
		default: // parent
			close(pipeFileDescr[1]);
			catchPkgVersionOutput(list);
	}
	close(pipeFileDescr[0]);
	close(pipeFileDescr[1]);
	return list;
}

void PortChecker::executePkgVersion()
{
	const char* index;
	if(isCompressedIndex())
	{
		index = handleCompressedIndex();
	}
	else
	{
		index = (preferences->getPreference(INDEX_PREF).c_str());
	}
	execl("/usr/sbin/pkg_version", "pkg_version", "-I", index, (char *) 0);
	perror("execl");
}

void PortChecker::catchPkgVersionOutput(QStringList* list)
{
	int n, i;
	char buff[BUFF_LEN] = {'\0'};
	string s;

	while((n = read(pipeFileDescr[0], buff, BUFF_LEN)) > 0)
	{
		for(i = 0; i < n; i++)
		{
			if(buff[i] == '\n')
			{
				if(s.find("<") != string::npos)
				{
					size_t endpos = s.find_last_not_of(" \t");
					s.erase(endpos);
					list->append(s.c_str());
				}
				s.clear();
			}
			else
			{
				s.append(1, buff[i]);
			}
		}
	}
}

bool PortChecker::isCompressedIndex()
{
	string index = preferences->getPreference(INDEX_PREF);
	if(index.find(".bz2", index.find_last_of("/")) != string::npos)
	{
		return TRUE;
	}
	return FALSE;
}

const char* PortChecker::handleCompressedIndex()
{
	char* home = getenv("HOME");
	string *index = new string(home);
	*index += "/.portracData";
	ostringstream oss("/bin/mkdir -p ", ios_base::app);
	oss << index->c_str();
	oss << " && /usr/bin/cd " << index->c_str();
	oss << " && /usr/bin/fetch -o - " << preferences->getPreference(INDEX_PREF).c_str();
	oss << " | /usr/bin/bunzip2 > " << index->c_str() << "/INDEX";
	system(oss.str().c_str());
	*index += "/INDEX";
	return index->c_str();
}

void PortChecker::run()
{
	list = check();
}

QStringList* PortChecker::getList()
{
	return list;
}
